package com.huimv.admin.controller;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.hikvision.cms.api.common.util.Digests;
import com.huimv.admin.common.utils.HttpClientSSLUtils;
import com.huimv.admin.common.utils.Result;
import com.huimv.admin.entity.hkwsdto.*;
import com.huimv.admin.service.ICameraAreaService;
import com.huimv.admin.service.ICameraBaseService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@CrossOrigin
@RestController
@RequestMapping("/admin/cameraHKVSSync")
public class CameraHKVSSync {

    @Autowired
    private ICameraAreaService cameraAreaService;

    @Autowired
    private ICameraBaseService cameraBaseService;

    @PostMapping("/sync")
    @Transactional
    public Result sync(@RequestBody Map<String, Integer> params) throws Exception {
        //获取默认控制中心  正康区域 的上级，暂时不用
//        System.out.println(testGetDefaultUnit());

        //根据中心UUID分页获取下级区  正康猪业id:1048576
        String unitUuid = testGetRegionsByUnitUuid(1048576);
        HkvsUnitDto hkvsUnitDto = JSONUtil.toBean(unitUuid, HkvsUnitDto.class);
        if (ObjectUtil.isNotEmpty(hkvsUnitDto) && hkvsUnitDto.getErrorCode() ==0 ){
            List<UnitDto1> list = hkvsUnitDto.getData().getList();
            if (ObjectUtil.isNotEmpty(list)){
                cameraAreaService.sync(params,list);
            }
        }else {
            return new Result(10001,hkvsUnitDto.getErrorMessage(),false);
        }
        //获取监控点
        String cameras = testGetCameras();
        HkvsCameraDto hkvsCameraDto = JSONUtil.toBean(cameras, HkvsCameraDto.class);
        if (ObjectUtil.isNotEmpty(hkvsCameraDto) && hkvsCameraDto.getErrorCode() ==0 ){
            List<CameraDto1> list = hkvsCameraDto.getData().getList();
            if (ObjectUtil.isNotEmpty(list)){
                cameraBaseService.sync(params,list);
            }
        }else {
            return new Result(10001,hkvsUnitDto.getErrorMessage(),false);
        }
        return new Result(10000,"同步成功",false);
    }

    /**
     * APPKEY需要到部署的平台服务器上生成。
     * <p>
     * 调用Openpai的操作码，需要去平台上生成，详见《海康威视iVMS-8700平台SDKV2.*.* HTTP-OpenAPI使用说明书.chm》中[获取AppKey和Secret]章节说明
     * </p>
     * <p>
     * 《海康威视iVMS-8700平台SDKV2.*.* HTTP-OpenAPI使用说明书.chm》 该文档请找技术支持或者交付的同事提供
     * </p>
     */
    private static final String APPKEY = "1c9ee657";

    /**
     * SECRET需要到部署的平台服务器上生成。
     * <p>
     * 调用Openpai的操作码，需要去平台上生成，详见《海康威视iVMS-8700平台SDKV2.*.* HTTP-OpenAPI使用说明书.chm》中[获取AppKey和Secret]章节说明
     * </p>
     * <p>
     * 《海康威视iVMS-8700平台SDKV2.*.* HTTP-OpenAPI使用说明书.chm》 该文档请找技术支持或者交付的同事提供
     * </p>
     */
    private static final String SECRET = "a5d723941c804a429447e71db2707ce8";


    /**
     * http请求地址
     * <p>openapi的地址,默认情况下openapi的IP端口与基础应用的IP端口是一致的</p>
     * <p>请将地址配置正确.</p>
     * <p>默认情况下是127.0.0.1:80 ，如果地址不通请根据实际情况修改IP端口</p>
     */
    private static final String OPENAPI_IP_PORT_HTTP = "http://61.153.248.106:100";
//    private static final String OPENAPI_IP_PORT_HTTP = "http://172.21.90:80";

    /**
     * https请求地址
     * <p>openapi的地址,默认情况下openapi的IP端口与基础应用的IP端口是一致的</p>
     * <p>请将地址配置正确.</p>
     * <p>默认情况下是127.0.0.1:443 ，如果地址不通请根据实际情况修改IP端口</p>
     */
    private static final String OPENAPI_IP_PORT_HTTPS = "https://61.153.248.106:4430";
//    private static final String OPENAPI_IP_PORT_HTTPS = "https://172.21.90:443";

    /**
     * 获取默认用户UUID的接口地址，此地址可以从《海康威视iVMS-8700平台SDKV2.*.* HTTP-OpenAPI使用说明书.chm》中具体的接口说明上获取
     */
    private static final String ITF_ADDRESS_GET_DEFAULT_USER_UUID = "/openapi/service/base/user/getDefaultUserUuid";

    /**
     * 分页获取监控点信息的接口地址，此地址可以从《海康威视iVMS-8700平台SDKV2.*.* HTTP-OpenAPI使用说明书.chm》中具体的接口说明上获取
     */
    private static final String ITF_ADDRESS_GET_CAMERAS = "/openapi/service/vss/res/getCameras";

    /**
     * 获取部门
     */
    private static final String ITF_ADDRESS_GET_DEPTINFOS = "/openapi/service/base/dept/getDeptInfos";

    /**
     * 获取网域
     */
    private static final String ITF_ADDRESS_GET_NETZONES = "/openapi/service/base/netZone/getNetZones";

    /**
     * 获取默认控制中心
     */
    private static final String ITF_ADDRESS_GET_DEFAULT_UNIT = "/openapi/service/base/org/getDefaultUnit";

    /**
     * 获取子系统
     */
    private static final String ITF_ADDRESS_GET_PLAT_SUBSYTEM = "/openapi/service/base/res/getPlatSubsytem";

    /**
     * 根据中心UUID分页获取下级区域
     */
    private static final String ITF_ADDRESS_GET_REGIONS_BY_UNIT_UUID = "/openapi/service/base/org/getRegionsByUnitUuid";

    /**
     * <p>操作用户UUID，即用户UUID，首次使用操作用户UUID可以通过接口 [获取默认用户UUID]来获取</p>
     * <p>也可以通过接口[分页获取用户]来获取</p>
     */
    private static final String OP_USER_UUID = "5b2eb534696b11e89c2e438f92627767";

    /**
     * 测试方法
     * @param args
     * @throws Exception
     */
    public static void main(String[] args) throws Exception {

        /***http方式调用***/
        //获取 默认用户id 初始化用
//        System.out.println(testGetDefaultUserUUID());

        //获取部门
//        System.out.println(testGetDeptInfos());
        //获取网域
//        System.out.println(testGetNetZones());
        //获取子系统
//        System.out.println(testGetPlatSubsytem());
        //获取默认控制中心
//        System.out.println(testGetDefaultUnit());
        //根据中心UUID分页获取下级区
//        System.out.println(testGetRegionsByUnitUuid(1048576));
        //获取监控点
        System.out.println(testGetCameras());
//        /***https方式调用***/
//        System.out.println(testGetDefaultUserUUID_Https());
//        System.out.println(testGetCameras_Https());
    }

    /**
     * HTTP方式
     * 获取子系统
     * @return
     * @throws Exception
     */
    private static String testGetRegionsByUnitUuid(Integer unitUuid) throws Exception{
        String url = OPENAPI_IP_PORT_HTTP + ITF_ADDRESS_GET_REGIONS_BY_UNIT_UUID;
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("appkey", APPKEY);//设置APPKEY
        map.put("time", System.currentTimeMillis());//设置时间参数
        map.put("pageNo", 1);//设置分页参数
        map.put("pageSize", 1000);//设置分页参数
        map.put("opUserUuid", OP_USER_UUID);//设置操作用户UUID
        map.put("parentUuid", unitUuid);// 根据父id查下面栋舍
        map.put("allChildren", 1);//获取子系统
        String params = JSON.toJSONString(map);
        System.out.println(" ====== getCameras请求参数：【" + params + "】");
        String data = HttpClientSSLUtils.doPost(url + "?token=" + Digests.buildToken(url + "?" + params, null, SECRET), params);
        System.out.println(" ====== getCameras请求返回结果：【{" + data + "}】");

        return data;
    }

    /**
     * HTTP方式
     * 获取默认控制中心
     * @return
     * @throws Exception
     */
    private static String testGetDefaultUnit() throws Exception{
        String url = OPENAPI_IP_PORT_HTTP + ITF_ADDRESS_GET_DEFAULT_UNIT;
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("appkey", APPKEY);//设置APPKEY
        map.put("time", System.currentTimeMillis());//设置时间参数
        map.put("pageNo", 1);//设置分页参数
        map.put("pageSize", 1000);//设置分页参数
        map.put("opUserUuid", OP_USER_UUID);//设置操作用户UUID
        map.put("subSystemCode", 1048576);//设置子系统编码  （视频）
        String params = JSON.toJSONString(map);
        System.out.println(" ====== getCameras请求参数：【" + params + "】");
        String data = HttpClientSSLUtils.doPost(url + "?token=" + Digests.buildToken(url + "?" + params, null, SECRET), params);
        System.out.println(" ====== getCameras请求返回结果：【{" + data + "}】");

        return data;
    }

    /**
     * HTTP方式
     * 获取子系统
     * @return
     * @throws Exception
     */
    private static String testGetPlatSubsytem() throws Exception{
        String url = OPENAPI_IP_PORT_HTTP + ITF_ADDRESS_GET_PLAT_SUBSYTEM;
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("appkey", APPKEY);//设置APPKEY
        map.put("time", System.currentTimeMillis());//设置时间参数
        map.put("pageNo", 1);//设置分页参数
        map.put("pageSize", 1000);//设置分页参数
        map.put("opUserUuid", OP_USER_UUID);//设置操作用户UUID
        String params = JSON.toJSONString(map);
        System.out.println(" ====== getCameras请求参数：【" + params + "】");
        String data = HttpClientSSLUtils.doPost(url + "?token=" + Digests.buildToken(url + "?" + params, null, SECRET), params);
        System.out.println(" ====== getCameras请求返回结果：【{" + data + "}】");

        return data;
    }


   /* private static String test() throws Exception{
        String baseUrl = "test.htpig.cn";
        String login = "/user/login";
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("user_name", APPKEY);//
        map.put("password", APPKEY);//
        String params = JSON.toJSONString(map);
        System.out.println(" ====== getCameras请求参数：【" + params + "】");
        String data = HttpClientSSLUtils.doGet("http://test.htpig.cn/rest/user/login?user_name=test&password=e10adc3949ba59abbe56e057f20f883e&db_name=MSSQL");
        System.out.println(" ====== getCameras请求返回结果：【{" + data + "}】");

        return data;
    }*/


    /**
     * HTTP方式
     * 获取默认用户UUID 测试
     * @return
     * @throws Exception
     */
    private static String testGetDefaultUserUUID() throws Exception{
        String url = OPENAPI_IP_PORT_HTTP + ITF_ADDRESS_GET_DEFAULT_USER_UUID;
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("appkey", APPKEY);//设置APPKEY
        map.put("time", System.currentTimeMillis());//设置时间参数
        String params =  JSON.toJSONString(map);
        System.out.println(" ====== testGetDefaultUserUUID 请求参数：【" + params + "】");
        String data = HttpClientSSLUtils.doPost(url + "?token=" + Digests.buildToken(url + "?" + params, null, SECRET), params);
        System.out.println(" ====== testGetDefaultUserUUID 请求返回结果：【{" + data + "}】");

        return data;
    }

    /**
     * HTTP方式
     * 分页获取监控点信息 测试
     * @return
     * @throws Exception
     */
    private static String testGetCameras() throws Exception{
        String url = OPENAPI_IP_PORT_HTTP + ITF_ADDRESS_GET_CAMERAS;
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("appkey", APPKEY);//设置APPKEY
        map.put("time", System.currentTimeMillis());//设置时间参数
        map.put("pageNo", 1);//设置分页参数
        map.put("pageSize", 1000);//设置分页参数
        map.put("opUserUuid", OP_USER_UUID);//设置操作用户UUID
        String params = JSON.toJSONString(map);
        System.out.println(" ====== getCameras请求参数：【" + params + "】");
        String data = HttpClientSSLUtils.doPost(url + "?token=" + Digests.buildToken(url + "?" + params, null, SECRET), params);
        System.out.println(" ====== getCameras请求返回结果：【{" + data + "}】");

        return data;
    }

    /**
     * HTTP方式
     * 分页获取部门 测试
     * @return
     * @throws Exception
     */
    private static String testGetDeptInfos() throws Exception{
        String url = OPENAPI_IP_PORT_HTTP + ITF_ADDRESS_GET_DEPTINFOS;
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("appkey", APPKEY);//设置APPKEY
        map.put("time", System.currentTimeMillis());//设置时间参数
        map.put("pageNo", 1);//设置分页参数
        map.put("pageSize", 1000);//设置分页参数
        map.put("opUserUuid", OP_USER_UUID);//设置操作用户UUID
        String params = JSON.toJSONString(map);
        System.out.println(" ====== getCameras请求参数：【" + params + "】");
        String data = HttpClientSSLUtils.doPost(url + "?token=" + Digests.buildToken(url + "?" + params, null, SECRET), params);
        System.out.println(" ====== getCameras请求返回结果：【{" + data + "}】");

        return data;
    }

    /**
     * HTTP方式
     * 分页获取网域 测试
     * @return
     * @throws Exception
     */
    private static String testGetNetZones() throws Exception{
        String url = OPENAPI_IP_PORT_HTTP + ITF_ADDRESS_GET_NETZONES;
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("appkey", APPKEY);//设置APPKEY
        map.put("time", System.currentTimeMillis());//设置时间参数
        map.put("pageNo", 1);//设置分页参数
        map.put("pageSize", 1000);//设置分页参数
        map.put("opUserUuid", OP_USER_UUID);//设置操作用户UUID
        String params = JSON.toJSONString(map);
        System.out.println(" ====== getCameras请求参数：【" + params + "】");
        String data = HttpClientSSLUtils.doPost(url + "?token=" + Digests.buildToken(url + "?" + params, null, SECRET), params);
        System.out.println(" ====== getCameras请求返回结果：【{" + data + "}】");

        return data;
    }

    /**
     * HTTPS方式
     * 获取默认用户UUID 测试
     * @return
     * @throws Exception
     */
    private static String testGetDefaultUserUUID_Https() throws Exception{
        String url = OPENAPI_IP_PORT_HTTPS + ITF_ADDRESS_GET_DEFAULT_USER_UUID;
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("appkey", APPKEY);//设置APPKEY
        map.put("time", System.currentTimeMillis());//设置时间参数
        String params =  JSON.toJSONString(map);
        System.out.println("============" + params + "============");
        String data = HttpClientSSLUtils.doPost(url + "?token=" + Digests.buildToken(url + "?" + params, null, SECRET), params);
        System.out.println("============" + data + "============");

        return data;
    }

    /**
     * HTTPS方式
     * 分页获取监控点信息 测试
     * @return
     * @throws Exception
     */
    private static String testGetCameras_Https() throws Exception{
        String url = OPENAPI_IP_PORT_HTTPS + ITF_ADDRESS_GET_CAMERAS;
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("appkey", APPKEY);//设置APPKEY
        map.put("time", System.currentTimeMillis());//设置时间参数
        map.put("pageNo", 1);//设置分页参数
        map.put("pageSize", 5);//设置分页参数
        map.put("opUserUuid", OP_USER_UUID);//设置操作用户UUID
        String params = JSON.toJSONString(map);
        System.out.println(" ====== getCameras请求参数：【" + params + "】");
        String data = HttpClientSSLUtils.doPost(url + "?token=" + Digests.buildToken(url + "?" + params, null, SECRET), params);
        System.out.println(" ====== getCameras请求返回结果：【{" + data + "}】");

        return data;
    }

}
