package com.huimv.admin.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.admin.common.utils.Result;
import com.huimv.admin.common.utils.ResultCode;
import com.huimv.admin.common.utils.ResultUtil;
import com.huimv.admin.entity.BasePigpen;
import com.huimv.admin.entity.CameraArea;
import com.huimv.admin.entity.CameraBase;
import com.huimv.admin.entity.CameraBrand;
import com.huimv.admin.entity.dto.CameraNameDto;
import com.huimv.admin.entity.hkwsdto.CameraDto1;
import com.huimv.admin.entity.hkwsdto.UnitDto1;
import com.huimv.admin.entity.vo.CameraListVo;
import com.huimv.admin.entity.vo.CameraListsVo;
import com.huimv.admin.entity.vo.CameraVo;
import com.huimv.admin.entity.vo.TreeBasePigpen;
import com.huimv.admin.mapper.CameraAreaMapper;
import com.huimv.admin.mapper.CameraBaseMapper;
import com.huimv.admin.mapper.CameraBrandMapper;
import com.huimv.admin.service.ICameraBaseService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-02-16
 */
@Service
public class CameraBaseServiceImpl extends ServiceImpl<CameraBaseMapper, CameraBase> implements ICameraBaseService {

    @Autowired
    private CameraAreaMapper cameraAreaMapper;
    @Autowired
    private CameraBrandMapper cameraBrandMapper;
    @Autowired
    private CameraBaseMapper cameraBaseMapper;

    @Override
    public Result listPage(CameraListVo cameraListVo) {
        Integer current = cameraListVo.getCurrent();
        String strchStr = cameraListVo.getStrchStr();
        Integer size = cameraListVo.getSize();
//        Integer areaId = cameraListVo.getAreaId();
        Integer sort = cameraListVo.getSort();
        Integer farmId = cameraListVo.getFarmId();
        Integer onLineStatus = cameraListVo.getOnLineStatus();
        String regionUuuid = cameraListVo.getRegionUuuid();
        QueryWrapper<CameraBase> wrapper = new QueryWrapper<>();
        if (null == current) {
            current = 1;
        }
        if (null == size) {
            size = 10;
        }
        if (farmId != null) {
            wrapper.eq("farm_id", farmId);
        }
        if (StringUtils.isNotBlank(regionUuuid)) {
            wrapper.eq("area_id", regionUuuid);
        }
        if (onLineStatus != null) {
            wrapper.eq("on_line_status", onLineStatus);
        }
        if (StringUtils.isNotBlank(strchStr)) {
            wrapper.and(i -> i.like("camera_name", strchStr));
        }
        if (sort == null) {
            wrapper.orderByDesc("id");
        } else if (sort == 1) {
            wrapper.orderByDesc("sort");
        } else {
            wrapper.orderByAsc("sort");
        }
        wrapper.orderByDesc("id");
        Page<CameraBase> page = page(new Page<CameraBase>(current, size), wrapper);
        return new Result(ResultCode.SUCCESS, page);
    }

    @Override
    public Result getCameraDetails(CameraListsVo cameraListsVo) {
        List<Integer> cameraIds = cameraListsVo.getCameraIds();
        CameraBrand cameraBrand = cameraBrandMapper.selectById(18);
        List list = new ArrayList();
        for (Integer cameraId : cameraIds) {
            CameraBase cameraBase = this.getById(cameraId);
            String regionUuid = cameraBase.getRegionUuid();
            CameraArea cameraArea = cameraAreaMapper.selectOne(new QueryWrapper<CameraArea>().eq("region_uuid", regionUuid));
            // http://61.153.248.106:6713/mag/hls/6bf5d2cf30bb440cad3fc126512f5188/1/live.m3u8
            String hls = "http://" + cameraArea.getPublicUrl() + cameraBrand.getStreamType() + cameraBase.getCameraUuid() + "/" + cameraBrand.getMainStream() + "/" + "live.m3u8";
            CameraVo cameraVo = new CameraVo();
            cameraVo.setHlsUrl(hls);
            cameraVo.setId(cameraBase.getId());
            cameraVo.setCameraName(cameraBase.getCameraName());
            list.add(cameraVo);
        }
        return ResultUtil.list(list);
    }


    @Override
    public void sync(Map<String, Integer> params, List<CameraDto1> list) {
        Integer farmId = params.get("farmId");

        List<CameraBase> huatongCameraBaseList = this.list(new QueryWrapper<CameraBase>().eq("farm_id", farmId));
        List<String> haikangCarmearId = list.stream().map(CameraDto1::getCameraUuid).collect(Collectors.toList());
        List<String> huatongCameraId = huatongCameraBaseList.stream().map(CameraBase::getCameraUuid).collect(Collectors.toList());
        CopyOnWriteArrayList<String> haikangDeviceIdCopy = ListUtil.toCopyOnWriteArrayList(haikangCarmearId);
        CopyOnWriteArrayList<String> huatongDeviceIdCopy = ListUtil.toCopyOnWriteArrayList(huatongCameraId);
        haikangDeviceIdCopy.removeAll(huatongCameraId);
        //TODO 想不出来，回头优化
        if (haikangDeviceIdCopy.size() > 0) {
            for (String s : haikangDeviceIdCopy) {
                for (CameraDto1 cameraDto1 : list) {
                    if (s.equals(cameraDto1.getCameraUuid())) {
                        CameraBase cameraBase = new CameraBase();
                        BeanUtil.copyProperties(cameraDto1, cameraBase);
                        CameraArea cameraArea = cameraAreaMapper.selectOne(new QueryWrapper<CameraArea>().eq("region_uuid", cameraDto1.getRegionUuid()));
                        if (ObjectUtil.isNotEmpty(cameraArea)) {
                            cameraBase.setAreaName(cameraArea.getAreaName());
                        }
                        cameraBase.setBrandName("海康威视");
                        cameraBase.setFarmId(farmId);
                        this.save(cameraBase);
                    }
                }
            }
        }
        //删除
        huatongDeviceIdCopy.removeAll(haikangCarmearId);
        if (huatongDeviceIdCopy.size() > 0) {
            this.remove(new QueryWrapper<CameraBase>().in("camera_uuid", huatongDeviceIdCopy));
        }

    }

    //更新摄像头状态
    @Override
    public void updateCameraOnlin(List<CameraDto1> list) {
        for (CameraDto1 cameraDto1 : list) {
            String cameraUuid = cameraDto1.getCameraUuid();
            CameraBase one = this.getOne(new QueryWrapper<CameraBase>().eq("camera_uuid", cameraUuid));
            one.setOnLineStatus(1 == cameraDto1.getOnLineStatus());
        }
    }

    @Override
    public Result list(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        QueryWrapper<CameraArea> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId);
        List<CameraArea> cameraAreas = cameraAreaMapper.selectList(queryWrapper);

        List<CameraNameDto> cameraNameDtoList = new LinkedList();
        for (CameraArea cameraArea : cameraAreas) {
            CameraNameDto cameraNameDto = new CameraNameDto();
            QueryWrapper<CameraBase> baseQueryWrapper = new QueryWrapper<>();
            baseQueryWrapper.eq("farm_id", farmId).eq("area_name", cameraArea.getAreaName());
            List<CameraBase> cameraBases = cameraBaseMapper.selectList(baseQueryWrapper);

            cameraNameDto.setCameraBases(cameraBases);
            cameraNameDto.setId(cameraArea.getId());
            cameraNameDto.setAreaName(cameraArea.getAreaName());

            cameraNameDtoList.add(cameraNameDto);


        }

        return new Result(ResultCode.SUCCESS, cameraNameDtoList);
    }


    /* *//**
     * 查询结果 转换成树形结构
     * @param bizBaseAreas 原始数据
     * @return 树
     *//*
    private List<CameraNameDto> parseBizBaseArea(List<CameraBase> bizBaseAreas){
        //构建需要展示的树形节点结构
        Map<String, CameraNameDto> nodeMap = bizBaseAreas.stream().map(basePigpen -> {
            CameraNameDto  baseVo = new CameraNameDto();
            baseVo.setId(basePigpen.getId()+"");
            baseVo.setCameraName(basePigpen.getCameraName());
            baseVo.setFarmId(basePigpen.getFarmId().toString());
            baseVo.setChildNode(new ArrayList<>());
            return baseVo;
        }).collect(Collectors.toMap(CameraNameDto::getId, b -> b,(k1, k2)->k1));
        //创建数组存父亲节点
        ArrayList<CameraNameDto> roots = new ArrayList<>();
        //构建树形
        nodeMap.values().forEach(item->{
            String parentCode = item.getParentId()+"";
            if (nodeMap.get(parentCode) == null){
                //父节点为空，说明当前节点就已经是父节点了 将该节点存起来
                roots.add(item);
            }else {
                //父节点 不为空，说明有父节点，拿到该节点的父节点的孩子节点（就是我自己),将该节点存起来
                nodeMap.get(parentCode).getChildNode().add(item);
            }
        });
        return roots;
    }*/
}
