package com.huimv.admin.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.admin.common.utils.DataUill;
import com.huimv.admin.common.utils.Result;
import com.huimv.admin.common.utils.ResultCode;
import com.huimv.admin.entity.BasePigpen;
import com.huimv.admin.entity.EnergyData;
import com.huimv.admin.entity.vo.*;
import com.huimv.admin.mapper.BasePigpenMapper;
import com.huimv.admin.mapper.EnergyDataMapper;
import com.huimv.admin.service.IEnergyDataService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.text.DecimalFormat;
import java.util.*;

/**
 * <p>
 * 耗能数据 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-02-14
 */
@Service
public class EnergyDataServiceImpl extends ServiceImpl<EnergyDataMapper, EnergyData> implements IEnergyDataService {

    @Autowired
    private EnergyDataMapper dataMapper;
    @Autowired
    private BasePigpenMapper basePigpenMapper;

    @Override
    public Result listCount(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        QueryWrapper<EnergyData> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId);
        EnergyDataVo energyDataVo = dataMapper.listCount(queryWrapper);
        Float f = 0.00f;
        f = (Float.parseFloat(energyDataVo.getElectricityValue()) +
                Float.parseFloat(energyDataVo.getFeedValue()) +
                Float.parseFloat(energyDataVo.getGasValue()) + Float.parseFloat(energyDataVo.getWaterValue()));
        DecimalFormat df1 = new DecimalFormat("#.00");
        energyDataVo.setCount(df1.format(f));
        return new Result(ResultCode.SUCCESS, energyDataVo);
    }

    @Override
    public Result listData(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String dataType = paramsMap.get("dataType");
        String timeType = paramsMap.get("timeType");
        String startTime = paramsMap.get("startTime");
        String endTime = paramsMap.get("endTime");

        if (dataType == null || dataType == "") {
            dataType = "1";
        }
        if (timeType == null || timeType == "") {
            timeType = "3";
        }

        QueryWrapper<EnergyData> dataQueryWrapper = new QueryWrapper<>();
        dataQueryWrapper.eq("farm_id", farmId);
        //用水
        if ("1".equals(dataType)) {
            //自定义
            if ("4".equals(timeType)) {
                startTime = startTime + " 00:00:00";
                endTime = endTime + " 23:59:59";
                dataQueryWrapper.between("create_time", startTime, endTime);
                return new Result(ResultCode.SUCCESS, dataMapper.listDayWater(dataQueryWrapper));
            }
            //全年
            if ("3".equals(timeType)) {
                DateTime dateTime = DateUtil.beginOfYear(new Date());
                dataQueryWrapper.ge("create_date", dateTime);
                List<EnergyAllDataVo> energyWaterDataVos = dataMapper.listYearWater(dataQueryWrapper);
                for (EnergyAllDataVo energyWaterDataVo : energyWaterDataVos) {
                    if (energyWaterDataVo.getValue() == null || energyWaterDataVo.getValue() == "") {
                        energyWaterDataVo.setValue("0");
                    }
                }
                return new Result(ResultCode.SUCCESS, energyWaterDataVos);
            }
            //本月
            if ("2".equals(timeType)) {
                Date timesMonthmorning = DataUill.getTimesMonthmorning();
                dataQueryWrapper.ge("create_date", timesMonthmorning);
                return new Result(ResultCode.SUCCESS, dataMapper.listDayWater(dataQueryWrapper));
            }
            //本周
            if ("1".equals(timeType)) {
                Calendar calendar = Calendar.getInstance();
                calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
                dataQueryWrapper.ge("create_date", calendar.getTime());
                List<EnergyAllDataVo> energyWaterDataVos = dataMapper.listDayWater(dataQueryWrapper);
                for (EnergyAllDataVo energyWaterDataVo : energyWaterDataVos) {
                    if (energyWaterDataVo.getValue() == null || energyWaterDataVo.getValue() == "") {
                        energyWaterDataVo.setValue("0");
                    }
                }
                return new Result(ResultCode.SUCCESS, energyWaterDataVos);
            }
        }
        //用电
        else if ("2".equals(dataType)) {
            //自定义
            if ("4".equals(timeType)) {
                startTime = startTime + " 00:00:00";
                endTime = endTime + " 23:59:59";
                dataQueryWrapper.between("create_time", startTime, endTime);
                return new Result(ResultCode.SUCCESS, dataMapper.listDayElectricity(dataQueryWrapper));
            }
            //全年
            if ("3".equals(timeType)) {
                DateTime dateTime = DateUtil.beginOfYear(new Date());
                dataQueryWrapper.ge("create_date", dateTime);
                List<EnergyAllDataVo> energyElectricityDataVos = dataMapper.listYearElectricity(dataQueryWrapper);
                for (EnergyAllDataVo energyElectricityDataVo : energyElectricityDataVos) {
                    if (energyElectricityDataVo.getValue() == null || energyElectricityDataVo.getValue() == "") {
                        energyElectricityDataVo.setValue("0");
                    }
                }
                return new Result(ResultCode.SUCCESS, energyElectricityDataVos);
            }
            //本月
            if ("2".equals(timeType)) {
                Date timesMonthmorning = DataUill.getTimesMonthmorning();
                dataQueryWrapper.ge("create_date", timesMonthmorning);
                return new Result(ResultCode.SUCCESS, dataMapper.listDayElectricity(dataQueryWrapper));
            }
            //本周
            if ("1".equals(timeType)) {
                Calendar calendar = Calendar.getInstance();
                calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
                dataQueryWrapper.ge("create_date", calendar.getTime());
                List<EnergyAllDataVo> energyElectricityDataVos = dataMapper.listDayElectricity(dataQueryWrapper);
                for (EnergyAllDataVo energyElectricityDataVo : energyElectricityDataVos) {
                    if (energyElectricityDataVo.getValue() == null || energyElectricityDataVo.getValue() == "") {
                        energyElectricityDataVo.setValue("0");
                    }
                }
                return new Result(ResultCode.SUCCESS, energyElectricityDataVos);
            }
        }
        //饲料
        else if ("3".equals(dataType)) {
            //自定义
            if ("4".equals(timeType)) {
                startTime = startTime + " 00:00:00";
                endTime = endTime + " 23:59:59";
                dataQueryWrapper.between("create_time", startTime, endTime);
                return new Result(ResultCode.SUCCESS, dataMapper.listDayFeed(dataQueryWrapper));
            }
            //全年
            if ("3".equals(timeType)) {
                DateTime dateTime = DateUtil.beginOfYear(new Date());
                dataQueryWrapper.ge("create_date", dateTime);
                List<EnergyAllDataVo> energyFeedDataVos = dataMapper.listYearFeed(dataQueryWrapper);
                for (EnergyAllDataVo energyFeedDataVo : energyFeedDataVos) {
                    if (energyFeedDataVo.getValue() == null || energyFeedDataVo.getValue() == "") {
                        energyFeedDataVo.setValue("0");
                    }
                }
                return new Result(ResultCode.SUCCESS, energyFeedDataVos);
            }
            //本月
            if ("2".equals(timeType)) {
                Date timesMonthmorning = DataUill.getTimesMonthmorning();
                dataQueryWrapper.ge("create_date", timesMonthmorning);
                return new Result(ResultCode.SUCCESS, dataMapper.listDayFeed(dataQueryWrapper));
            }
            //本周
            if ("1".equals(timeType)) {
                Calendar calendar = Calendar.getInstance();
                calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
                dataQueryWrapper.ge("create_date", calendar.getTime());
                List<EnergyAllDataVo> energyFeedDataVos = dataMapper.listDayFeed(dataQueryWrapper);
                for (EnergyAllDataVo energyFeedDataVo : energyFeedDataVos) {
                    if (energyFeedDataVo.getValue() == null || energyFeedDataVo.getValue() == "") {
                        energyFeedDataVo.setValue("0");
                    }
                }
                return new Result(ResultCode.SUCCESS, energyFeedDataVos);
            }
        }
        //天然气
        else if ("4".equals(dataType)) {
            //自定义
            if ("4".equals(timeType)) {
                startTime = startTime + " 00:00:00";
                endTime = endTime + " 23:59:59";
                dataQueryWrapper.between("create_time", startTime, endTime);
                return new Result(ResultCode.SUCCESS, dataMapper.listDayGas(dataQueryWrapper));
            }
            //全年
            if ("3".equals(timeType)) {
                DateTime dateTime = DateUtil.beginOfYear(new Date());
                dataQueryWrapper.ge("create_date", dateTime);
                List<EnergyAllDataVo> energyGasDataVos = dataMapper.listYearGas(dataQueryWrapper);
                for (EnergyAllDataVo energyGasDataVo : energyGasDataVos) {
                    if (energyGasDataVo.getValue() == null || energyGasDataVo.getValue() == "") {
                        energyGasDataVo.setValue("0");
                    }
                }
                return new Result(ResultCode.SUCCESS, energyGasDataVos);
            }
            //本月
            if ("2".equals(timeType)) {
                Date timesMonthmorning = DataUill.getTimesMonthmorning();
                dataQueryWrapper.ge("create_date", timesMonthmorning);
                return new Result(ResultCode.SUCCESS, dataMapper.listDayGas(dataQueryWrapper));
            }
            //本周
            if ("1".equals(timeType)) {
                Calendar calendar = Calendar.getInstance();
                calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
                dataQueryWrapper.ge("create_date", calendar.getTime());
                List<EnergyAllDataVo> energyGasDataVos = dataMapper.listDayGas(dataQueryWrapper);
                for (EnergyAllDataVo energyGasDataVo : energyGasDataVos) {
                    if (energyGasDataVo.getValue() == null || energyGasDataVo.getValue() == "") {
                        energyGasDataVo.setValue("0");
                    }
                }
                return new Result(ResultCode.SUCCESS, energyGasDataVos);
            }
        }
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result listRank(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String dataType = paramsMap.get("dataType");
        String timeType = paramsMap.get("timeType");
        String startTime = paramsMap.get("startTime");
        String endTime = paramsMap.get("endTime");
        if (dataType == null || dataType == "") {
            dataType = "1";
        }
        if (timeType == null || timeType == "") {
            timeType = "3";
        }
        QueryWrapper<BasePigpen> basePigpenQueryWrapper = new QueryWrapper<>();
        basePigpenQueryWrapper.eq("farm_id", farmId);
        JSONArray jsonArray = new JSONArray();

        //用水
        if ("1".equals(dataType)) {
            //自定义
            if ("4".equals(timeType)) {
                startTime = startTime + " 00:00:00";
                endTime = endTime + " 23:59:59";
                basePigpenQueryWrapper.eq("parent_id", 0);
                Float value = 0.00f;
                List<BasePigpen> basePigpens = basePigpenMapper.selectList(basePigpenQueryWrapper);//得到所有的栋舍
                QueryWrapper<BasePigpen> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("farm_id",farmId).in("other2",basePigpens);
                List<BasePigpen> basePigpens1 = basePigpenMapper.selectList(queryWrapper);//得到所有下属的楼层和单元
                System.out.println(basePigpens1);
                for (BasePigpen basePigpen : basePigpens1) { //查找数据，到单元层次
                    QueryWrapper<EnergyData> dataQueryWrapper = new QueryWrapper<>();
                    dataQueryWrapper.eq("farm_id", farmId).eq("unit_id", basePigpen.getId())
                            .between("create_date", startTime, endTime);
                    dataQueryWrapper.select(
                            "IFNULL(SUM(water_value),'0') waterValue  "
                    );
                    EnergyData energyData = dataMapper.selectOne(dataQueryWrapper);
                    if (ObjectUtil.isNotEmpty(energyData)) {
                        value = value + Float.parseFloat(energyData.getWaterValue());
                        QueryWrapper<BasePigpen> queryWrapper1 = new QueryWrapper<>();
                        queryWrapper1.eq("id", basePigpen.getParentId()).eq("farm_id", farmId);
                        BasePigpen basePigpen1 = basePigpenMapper.selectOne(queryWrapper1);
                        JSONObject jsonObject = new JSONObject();
                        jsonObject.put("value", value);
                        jsonObject.put("buildName", basePigpen1.getBuildName());
                        jsonArray.add(jsonObject);
                        value = 0.00f;
                    }
                }
               /* for (BasePigpen basePigpen : basePigpens) {
                    QueryWrapper<BasePigpen> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("farm_id", farmId).eq("parent_id", basePigpen.getId()).eq("f_type", 2);
                    List<BasePigpen> pigpens = basePigpenMapper.selectList(queryWrapper);//得到所有的楼层
                    Float value = 0.00f;
                    for (BasePigpen pigpen : pigpens) {
                        JSONObject jsonObject = new JSONObject();
                        QueryWrapper<BasePigpen> queryWrapper1 = new QueryWrapper<>();
                        queryWrapper1.eq("farm_id", farmId).eq("parent_id", pigpen.getId());
                        List<BasePigpen> basePigpens1 = basePigpenMapper.selectList(queryWrapper1);//得到所有的单元

                        for (BasePigpen basePigpen1 : basePigpens1) {
                            QueryWrapper<EnergyData> dataQueryWrapper = new QueryWrapper<>();
                            dataQueryWrapper.eq("farm_id", farmId).eq("unit_id", basePigpen1.getId())
                                    .between("create_date", startTime, endTime);
                            dataQueryWrapper.select(
                                    "IFNULL(SUM(water_value),0) waterValue  "
                            );
                            EnergyData energyData = dataMapper.selectOne(dataQueryWrapper);
                            value = value + Float.parseFloat(energyData.getWaterValue()); //把所有单元的值相加
                        }
                        jsonObject.put("value", value);
                        jsonObject.put("buildName", pigpen.getBuildName());
                        jsonArray.add(jsonObject);
                        value = 0.00f;
                    }
                }*/
                JSONArray arraySort = jsonArraySort(jsonArray, "value", true);
                return new Result(ResultCode.SUCCESS, arraySort);
            }
            //全年
            else if ("3".equals(timeType)) {
                DateTime dateTime = DateUtil.beginOfYear(new Date());
                basePigpenQueryWrapper.eq("parent_id", 0);
                Float value = 0.00f;
                List<BasePigpen> basePigpens = basePigpenMapper.selectList(basePigpenQueryWrapper);//得到所有的栋舍
                for (BasePigpen basePigpen : basePigpens) {
                    QueryWrapper<BasePigpen> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("farm_id",farmId).like("other2",basePigpen.getId());
                    List<BasePigpen> selectList = basePigpenMapper.selectList(queryWrapper);//所有下属
                    for (BasePigpen pigpen : selectList) {
                        QueryWrapper<EnergyData> dataQueryWrapper = new QueryWrapper<>();
                        dataQueryWrapper.eq("farm_id", farmId).eq("unit_id", pigpen.getId())
                                .ge("create_date", dateTime);
                        dataQueryWrapper.select(
                                "IFNULL(SUM(water_value),'0') waterValue  "
                        );
                        EnergyData energyData = dataMapper.selectOne(dataQueryWrapper);
                        if (Float.parseFloat(energyData.getWaterValue())!=0.0f) {
                            value = value + Float.parseFloat(energyData.getWaterValue());
                            QueryWrapper<BasePigpen> queryWrapper1 = new QueryWrapper<>();
                            queryWrapper1.eq("id", pigpen.getParentId()).eq("farm_id", farmId);
                            BasePigpen basePigpen1 = basePigpenMapper.selectOne(queryWrapper1);
                            JSONObject jsonObject = new JSONObject();
                            jsonObject.put("value", value);
                            jsonObject.put("buildName", basePigpen1.getBuildName());
                            jsonArray.add(jsonObject);
                            value = 0.00f;
                        }
                    }

                }

                JSONArray arraySort = jsonArraySort(jsonArray, "value", true);
                return new Result(ResultCode.SUCCESS, arraySort);
            }

            //本月
            else if ("2".equals(timeType)) {
                Date timesMonthmorning = DataUill.getTimesMonthmorning();
                basePigpenQueryWrapper.eq("parent_id", 0);
                List<BasePigpen> basePigpens = basePigpenMapper.selectList(basePigpenQueryWrapper);//得到所有的栋舍
                for (BasePigpen basePigpen : basePigpens) {
                    QueryWrapper<BasePigpen> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("farm_id", farmId).eq("parent_id", basePigpen.getId()).eq("f_type", 2);
                    List<BasePigpen> pigpens = basePigpenMapper.selectList(queryWrapper);//得到所有的楼层
                    Float value = 0.00f;
                    for (BasePigpen pigpen : pigpens) {
                        JSONObject jsonObject = new JSONObject();
                        QueryWrapper<BasePigpen> queryWrapper1 = new QueryWrapper<>();
                        queryWrapper1.eq("farm_id", farmId).eq("parent_id", pigpen.getId());
                        List<BasePigpen> basePigpens1 = basePigpenMapper.selectList(queryWrapper1);//得到所有的单元

                        for (BasePigpen basePigpen1 : basePigpens1) {
                            QueryWrapper<EnergyData> dataQueryWrapper = new QueryWrapper<>();
                            dataQueryWrapper.eq("farm_id", farmId).eq("unit_id", basePigpen1.getId())
                                    .ge("create_date", timesMonthmorning);
                            dataQueryWrapper.select(
                                    "IFNULL(SUM(water_value),0) waterValue  "
                            );
                            EnergyData energyData = dataMapper.selectOne(dataQueryWrapper);
                            value = value + Float.parseFloat(energyData.getWaterValue()); //把所有单元的值相加
                        }
                        jsonObject.put("value", value);
                        jsonObject.put("buildName", pigpen.getBuildName());
                        jsonArray.add(jsonObject);
                        value = 0.00f;
                    }
                }
                JSONArray arraySort = jsonArraySort(jsonArray, "value", true);
                return new Result(ResultCode.SUCCESS, arraySort);
            }
            //本周
            else if ("1".equals(timeType)) {
                Calendar calendar = Calendar.getInstance();
                calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
                basePigpenQueryWrapper.eq("parent_id", 0);
                List<BasePigpen> basePigpens = basePigpenMapper.selectList(basePigpenQueryWrapper);//得到所有的栋舍
                for (BasePigpen basePigpen : basePigpens) {
                    QueryWrapper<BasePigpen> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("farm_id", farmId).eq("parent_id", basePigpen.getId()).eq("f_type", 2);
                    List<BasePigpen> pigpens = basePigpenMapper.selectList(queryWrapper);//得到所有的楼层
                    Float value = 0.00f;
                    for (BasePigpen pigpen : pigpens) {
                        JSONObject jsonObject = new JSONObject();
                        QueryWrapper<BasePigpen> queryWrapper1 = new QueryWrapper<>();
                        queryWrapper1.eq("farm_id", farmId).eq("parent_id", pigpen.getId());
                        List<BasePigpen> basePigpens1 = basePigpenMapper.selectList(queryWrapper1);//得到所有的单元

                        for (BasePigpen basePigpen1 : basePigpens1) {
                            QueryWrapper<EnergyData> dataQueryWrapper = new QueryWrapper<>();
                            dataQueryWrapper.eq("farm_id", farmId).eq("unit_id", basePigpen1.getId())
                                    .ge("create_date", calendar.getTime());
                            dataQueryWrapper.select(
                                    "IFNULL(SUM(water_value),0) waterValue  "
                            );
                            EnergyData energyData = dataMapper.selectOne(dataQueryWrapper);
                            value = value + Float.parseFloat(energyData.getWaterValue()); //把所有单元的值相加
                        }
                        jsonObject.put("value", value);
                        jsonObject.put("buildName", pigpen.getBuildName());
                        jsonArray.add(jsonObject);
                        value = 0.00f;
                    }
                }
                JSONArray arraySort = jsonArraySort(jsonArray, "value", true);
                return new Result(ResultCode.SUCCESS, arraySort);
            }
        }
        //用电
        else if ("2".equals(dataType)) {
            //自定义
            if ("4".equals(timeType)) {
                startTime = startTime + " 00:00:00";
                endTime = endTime + " 23:59:59";
                basePigpenQueryWrapper.eq("parent_id", 0);
                List<BasePigpen> basePigpens = basePigpenMapper.selectList(basePigpenQueryWrapper);//得到所有的栋舍
                for (BasePigpen basePigpen : basePigpens) {
                    QueryWrapper<BasePigpen> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("farm_id", farmId).eq("parent_id", basePigpen.getId()).eq("f_type", 2);
                    List<BasePigpen> pigpens = basePigpenMapper.selectList(queryWrapper);//得到所有的楼层
                    Float value = 0.00f;
                    for (BasePigpen pigpen : pigpens) {
                        JSONObject jsonObject = new JSONObject();
                        QueryWrapper<BasePigpen> queryWrapper1 = new QueryWrapper<>();
                        queryWrapper1.eq("farm_id", farmId).eq("parent_id", pigpen.getId());
                        List<BasePigpen> basePigpens1 = basePigpenMapper.selectList(queryWrapper1);//得到所有的单元

                        for (BasePigpen basePigpen1 : basePigpens1) {
                            QueryWrapper<EnergyData> dataQueryWrapper = new QueryWrapper<>();
                            dataQueryWrapper.eq("farm_id", farmId).eq("unit_id", basePigpen1.getId())
                                    .between("create_date", startTime, endTime);
                            dataQueryWrapper.select(
                                    "IFNULL(SUM(electricity_value),0) electricityValue  "
                            );
                            EnergyData energyData = dataMapper.selectOne(dataQueryWrapper);
                            value = value + Float.parseFloat(energyData.getElectricityValue()); //把所有单元的值相加
                        }
                        jsonObject.put("value", value);
                        jsonObject.put("buildName", pigpen.getBuildName());
                        jsonArray.add(jsonObject);
                        value = 0.00f;
                    }
                }
                JSONArray arraySort = jsonArraySort(jsonArray, "value", true);
                return new Result(ResultCode.SUCCESS, arraySort);
            }
            //全年
            else if ("3".equals(timeType)) {
                DateTime dateTime = DateUtil.beginOfYear(new Date());
                basePigpenQueryWrapper.eq("parent_id", 0);
                List<BasePigpen> basePigpens = basePigpenMapper.selectList(basePigpenQueryWrapper);//得到所有的栋舍
                for (BasePigpen basePigpen : basePigpens) {
                    QueryWrapper<BasePigpen> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("farm_id", farmId).eq("parent_id", basePigpen.getId()).eq("f_type", 2);
                    List<BasePigpen> pigpens = basePigpenMapper.selectList(queryWrapper);//得到所有的楼层
                    Float value = 0.00f;
                    for (BasePigpen pigpen : pigpens) {
                        JSONObject jsonObject = new JSONObject();
                        QueryWrapper<BasePigpen> queryWrapper1 = new QueryWrapper<>();
                        queryWrapper1.eq("farm_id", farmId).eq("parent_id", pigpen.getId());
                        List<BasePigpen> basePigpens1 = basePigpenMapper.selectList(queryWrapper1);//得到所有的单元

                        for (BasePigpen basePigpen1 : basePigpens1) {
                            QueryWrapper<EnergyData> dataQueryWrapper = new QueryWrapper<>();
                            dataQueryWrapper.eq("farm_id", farmId).eq("unit_id", basePigpen1.getId())
                                    .ge("create_date", dateTime);
                            dataQueryWrapper.select(
                                    "IFNULL(SUM(electricity_value),0) electricityValue  "
                            );
                            EnergyData energyData = dataMapper.selectOne(dataQueryWrapper);
                            value = value + Float.parseFloat(energyData.getElectricityValue()); //把所有单元的值相加
                        }
                        jsonObject.put("value", value);
                        jsonObject.put("buildName", pigpen.getBuildName());
                        jsonArray.add(jsonObject);
                        value = 0.00f;
                    }
                }
                JSONArray arraySort = jsonArraySort(jsonArray, "value", true);
                return new Result(ResultCode.SUCCESS, arraySort);
            }

            //本月
            else if ("2".equals(timeType)) {
                Date timesMonthmorning = DataUill.getTimesMonthmorning();
                basePigpenQueryWrapper.eq("parent_id", 0);
                List<BasePigpen> basePigpens = basePigpenMapper.selectList(basePigpenQueryWrapper);//得到所有的栋舍
                for (BasePigpen basePigpen : basePigpens) {
                    QueryWrapper<BasePigpen> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("farm_id", farmId).eq("parent_id", basePigpen.getId()).eq("f_type", 2);
                    List<BasePigpen> pigpens = basePigpenMapper.selectList(queryWrapper);//得到所有的楼层
                    Float value = 0.00f;
                    for (BasePigpen pigpen : pigpens) {
                        JSONObject jsonObject = new JSONObject();
                        QueryWrapper<BasePigpen> queryWrapper1 = new QueryWrapper<>();
                        queryWrapper1.eq("farm_id", farmId).eq("parent_id", pigpen.getId());
                        List<BasePigpen> basePigpens1 = basePigpenMapper.selectList(queryWrapper1);//得到所有的单元

                        for (BasePigpen basePigpen1 : basePigpens1) {
                            QueryWrapper<EnergyData> dataQueryWrapper = new QueryWrapper<>();
                            dataQueryWrapper.eq("farm_id", farmId).eq("unit_id", basePigpen1.getId())
                                    .ge("create_date", timesMonthmorning);
                            dataQueryWrapper.select(
                                    "IFNULL(SUM(electricity_value),0) electricityValue  "
                            );
                            EnergyData energyData = dataMapper.selectOne(dataQueryWrapper);
                            value = value + Float.parseFloat(energyData.getElectricityValue()); //把所有单元的值相加
                        }
                        jsonObject.put("value", value);
                        jsonObject.put("buildName", pigpen.getBuildName());
                        jsonArray.add(jsonObject);
                        value = 0.00f;
                    }
                }
                JSONArray arraySort = jsonArraySort(jsonArray, "value", true);
                return new Result(ResultCode.SUCCESS, arraySort);
            }
            //本周
            else if ("1".equals(timeType)) {
                Calendar calendar = Calendar.getInstance();
                calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
                basePigpenQueryWrapper.eq("parent_id", 0);
                List<BasePigpen> basePigpens = basePigpenMapper.selectList(basePigpenQueryWrapper);//得到所有的栋舍
                for (BasePigpen basePigpen : basePigpens) {
                    QueryWrapper<BasePigpen> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("farm_id", farmId).eq("parent_id", basePigpen.getId()).eq("f_type", 2);
                    List<BasePigpen> pigpens = basePigpenMapper.selectList(queryWrapper);//得到所有的楼层
                    Float value = 0.00f;
                    for (BasePigpen pigpen : pigpens) {
                        JSONObject jsonObject = new JSONObject();
                        QueryWrapper<BasePigpen> queryWrapper1 = new QueryWrapper<>();
                        queryWrapper1.eq("farm_id", farmId).eq("parent_id", pigpen.getId());
                        List<BasePigpen> basePigpens1 = basePigpenMapper.selectList(queryWrapper1);//得到所有的单元

                        for (BasePigpen basePigpen1 : basePigpens1) {
                            QueryWrapper<EnergyData> dataQueryWrapper = new QueryWrapper<>();
                            dataQueryWrapper.eq("farm_id", farmId).eq("unit_id", basePigpen1.getId())
                                    .ge("create_date", calendar.getTime());
                            dataQueryWrapper.select(
                                    "IFNULL(SUM(electricity_value),0) electricityValue  "
                            );
                            EnergyData energyData = dataMapper.selectOne(dataQueryWrapper);
                            value = value + Float.parseFloat(energyData.getElectricityValue()); //把所有单元的值相加
                        }
                        jsonObject.put("value", value);
                        jsonObject.put("buildName", pigpen.getBuildName());
                        jsonArray.add(jsonObject);
                        value = 0.00f;
                    }
                }
                JSONArray arraySort = jsonArraySort(jsonArray, "value", true);
                return new Result(ResultCode.SUCCESS, arraySort);
            }
        }
        //饲料
        else if ("3".equals(dataType)) {
            //自定义
            if ("4".equals(timeType)) {
                startTime = startTime + " 00:00:00";
                endTime = endTime + " 23:59:59";
                basePigpenQueryWrapper.eq("parent_id", 0);
                List<BasePigpen> basePigpens = basePigpenMapper.selectList(basePigpenQueryWrapper);//得到所有的栋舍
                for (BasePigpen basePigpen : basePigpens) {
                    QueryWrapper<BasePigpen> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("farm_id", farmId).like("other2",basePigpen.getId());
                    List<BasePigpen> pigpens = basePigpenMapper.selectList(queryWrapper);//得到所有的楼层
                    Float value = 0.00f;
                    for (BasePigpen pigpen : pigpens) {
                        JSONObject jsonObject = new JSONObject();
                        QueryWrapper<EnergyData> dataQueryWrapper = new QueryWrapper<>();
                        dataQueryWrapper.eq("farm_id", farmId).eq("unit_id", pigpen.getId())
                                .between("create_date", startTime,endTime);
                        dataQueryWrapper.select(
                                "IFNULL(SUM(feed_value),0) feedValue  "
                        );
                        EnergyData energyData = dataMapper.selectOne(dataQueryWrapper);
                        if (Float.parseFloat(energyData.getFeedValue())!=0.0f) {
                            value = value + Float.parseFloat(energyData.getFeedValue());
                            QueryWrapper<BasePigpen> queryWrapper1 = new QueryWrapper<>();
                            queryWrapper1.eq("id", pigpen.getParentId()).eq("farm_id", farmId);
                            BasePigpen basePigpen1 = basePigpenMapper.selectOne(queryWrapper1);
                            jsonObject.put("value", value);
                            jsonObject.put("buildName", basePigpen1.getBuildName());
                            jsonArray.add(jsonObject);
                            value = 0.00f;
                        }
                    }
                }
                JSONArray arraySort = jsonArraySort(jsonArray, "value", true);
                return new Result(ResultCode.SUCCESS, arraySort);
            }
            //全年
            else if ("3".equals(timeType)) {
                DateTime dateTime = DateUtil.beginOfYear(new Date());
                basePigpenQueryWrapper.eq("parent_id", 0);
                List<BasePigpen> basePigpens = basePigpenMapper.selectList(basePigpenQueryWrapper);//得到所有的栋舍
                for (BasePigpen basePigpen : basePigpens) {
                    QueryWrapper<BasePigpen> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("farm_id", farmId).like("other2",basePigpen.getId());
                    List<BasePigpen> pigpens = basePigpenMapper.selectList(queryWrapper);//得到所有的楼层
                    Float value = 0.00f;
                    for (BasePigpen pigpen : pigpens) {
                        JSONObject jsonObject = new JSONObject();
                        QueryWrapper<EnergyData> dataQueryWrapper = new QueryWrapper<>();
                        dataQueryWrapper.eq("farm_id", farmId).eq("unit_id", pigpen.getId())
                                .ge("create_date", dateTime);
                        dataQueryWrapper.select(
                                "IFNULL(SUM(feed_value),0) feedValue  "
                        );
                        EnergyData energyData = dataMapper.selectOne(dataQueryWrapper);
                        if (Float.parseFloat(energyData.getFeedValue())!=0.0f) {
                            value = value + Float.parseFloat(energyData.getFeedValue());
                            QueryWrapper<BasePigpen> queryWrapper1 = new QueryWrapper<>();
                            queryWrapper1.eq("id", pigpen.getParentId()).eq("farm_id", farmId);
                            BasePigpen basePigpen1 = basePigpenMapper.selectOne(queryWrapper1);
                            jsonObject.put("value", value);
                            jsonObject.put("buildName", basePigpen1.getBuildName());
                            jsonArray.add(jsonObject);
                            value = 0.00f;
                        }
                    }
                }
                JSONArray arraySort = jsonArraySort(jsonArray, "value", true);
                return new Result(ResultCode.SUCCESS, arraySort);
            }
            //本月
            else if ("2".equals(timeType)) {
                Date timesMonthmorning = DataUill.getTimesMonthmorning();
                basePigpenQueryWrapper.eq("parent_id", 0);
                List<BasePigpen> basePigpens = basePigpenMapper.selectList(basePigpenQueryWrapper);//得到所有的栋舍
                for (BasePigpen basePigpen : basePigpens) {
                    QueryWrapper<BasePigpen> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("farm_id", farmId).like("other2",basePigpen.getId());
                    List<BasePigpen> pigpens = basePigpenMapper.selectList(queryWrapper);//得到所有的楼层
                    Float value = 0.00f;
                    for (BasePigpen pigpen : pigpens) {
                        JSONObject jsonObject = new JSONObject();
                        QueryWrapper<EnergyData> dataQueryWrapper = new QueryWrapper<>();
                        dataQueryWrapper.eq("farm_id", farmId).eq("unit_id", pigpen.getId())
                                .ge("create_date",timesMonthmorning);
                        dataQueryWrapper.select(
                                "IFNULL(SUM(feed_value),0) feedValue  "
                        );
                        EnergyData energyData = dataMapper.selectOne(dataQueryWrapper);
                        if (Float.parseFloat(energyData.getFeedValue())!=0.0f) {
                            value = value + Float.parseFloat(energyData.getFeedValue());
                            QueryWrapper<BasePigpen> queryWrapper1 = new QueryWrapper<>();
                            queryWrapper1.eq("id", pigpen.getParentId()).eq("farm_id", farmId);
                            BasePigpen basePigpen1 = basePigpenMapper.selectOne(queryWrapper1);
                            jsonObject.put("value", value);
                            jsonObject.put("buildName", basePigpen1.getBuildName());
                            jsonArray.add(jsonObject);
                            value = 0.00f;
                        }
                    }
                }
                JSONArray arraySort = jsonArraySort(jsonArray, "value", true);
                return new Result(ResultCode.SUCCESS, arraySort);
            }
            //本周
            else if ("1".equals(timeType)) {
                Calendar calendar = Calendar.getInstance();
                calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
                basePigpenQueryWrapper.eq("parent_id", 0);
                List<BasePigpen> basePigpens = basePigpenMapper.selectList(basePigpenQueryWrapper);//得到所有的栋舍
                for (BasePigpen basePigpen : basePigpens) {
                    QueryWrapper<BasePigpen> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("farm_id", farmId).like("other2",basePigpen.getId());
                    List<BasePigpen> pigpens = basePigpenMapper.selectList(queryWrapper);//得到所有的楼层
                    Float value = 0.00f;
                    for (BasePigpen pigpen : pigpens) {
                        JSONObject jsonObject = new JSONObject();
                        QueryWrapper<EnergyData> dataQueryWrapper = new QueryWrapper<>();
                        dataQueryWrapper.eq("farm_id", farmId).eq("unit_id", pigpen.getId())
                                .ge("create_date",calendar.getTime());
                        dataQueryWrapper.select(
                                "IFNULL(SUM(feed_value),0) feedValue  "
                        );
                        EnergyData energyData = dataMapper.selectOne(dataQueryWrapper);
                        if (Float.parseFloat(energyData.getFeedValue())!=0.0f) {
                            value = value + Float.parseFloat(energyData.getFeedValue());
                            QueryWrapper<BasePigpen> queryWrapper1 = new QueryWrapper<>();
                            queryWrapper1.eq("id", pigpen.getParentId()).eq("farm_id", farmId);
                            BasePigpen basePigpen1 = basePigpenMapper.selectOne(queryWrapper1);
                            jsonObject.put("value", value);
                            jsonObject.put("buildName", basePigpen1.getBuildName());
                            jsonArray.add(jsonObject);
                            value = 0.00f;
                        }
                    }
                }
                JSONArray arraySort = jsonArraySort(jsonArray, "value", true);
                return new Result(ResultCode.SUCCESS, arraySort);
            }
        }
        //天然气
        else if ("4".equals(dataType)) {
            //自定义
            if ("4".equals(timeType)) {
                startTime = startTime + " 00:00:00";
                endTime = endTime + " 23:59:59";
                basePigpenQueryWrapper.eq("parent_id", 0);
                List<BasePigpen> basePigpens = basePigpenMapper.selectList(basePigpenQueryWrapper);//得到所有的栋舍
                for (BasePigpen basePigpen : basePigpens) {
                    QueryWrapper<BasePigpen> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("farm_id", farmId).like("other2",basePigpen.getId());
                    List<BasePigpen> pigpens = basePigpenMapper.selectList(queryWrapper);//得到所有的楼层
                    Float value = 0.00f;
                    for (BasePigpen pigpen : pigpens) {
                        JSONObject jsonObject = new JSONObject();
                        QueryWrapper<EnergyData> dataQueryWrapper = new QueryWrapper<>();
                        dataQueryWrapper.eq("farm_id", farmId).eq("unit_id", pigpen.getId())
                                .between("create_date", startTime,endTime);
                        dataQueryWrapper.select(
                                "IFNULL(SUM(gas_value),0) gasValue  "
                        );
                        EnergyData energyData = dataMapper.selectOne(dataQueryWrapper);
                        if (Float.parseFloat(energyData.getGasValue())!=0.0f) {
                            value = value + Float.parseFloat(energyData.getGasValue());
                            QueryWrapper<BasePigpen> queryWrapper1 = new QueryWrapper<>();
                            queryWrapper1.eq("id", pigpen.getParentId()).eq("farm_id", farmId);
                            BasePigpen basePigpen1 = basePigpenMapper.selectOne(queryWrapper1);
                            jsonObject.put("value", value);
                            jsonObject.put("buildName", basePigpen1.getBuildName());
                            jsonArray.add(jsonObject);
                            value = 0.00f;
                        }
                    }
                }
                JSONArray arraySort = jsonArraySort(jsonArray, "value", true);
                return new Result(ResultCode.SUCCESS, arraySort);
            }
            //全年
            else if ("3".equals(timeType)) {
                DateTime dateTime = DateUtil.beginOfYear(new Date());
                basePigpenQueryWrapper.eq("parent_id", 0);
                List<BasePigpen> basePigpens = basePigpenMapper.selectList(basePigpenQueryWrapper);//得到所有的栋舍
                for (BasePigpen basePigpen : basePigpens) {
                    QueryWrapper<BasePigpen> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("farm_id", farmId).like("other2",basePigpen.getId());
                    List<BasePigpen> pigpens = basePigpenMapper.selectList(queryWrapper);//得到所有的楼层
                    Float value = 0.00f;
                    for (BasePigpen pigpen : pigpens) {
                        JSONObject jsonObject = new JSONObject();
                        QueryWrapper<EnergyData> dataQueryWrapper = new QueryWrapper<>();
                        dataQueryWrapper.eq("farm_id", farmId).eq("unit_id", pigpen.getId())
                                .ge("create_date", dateTime);
                        dataQueryWrapper.select(
                                "IFNULL(SUM(gas_value),0) gasValue  "
                        );
                        EnergyData energyData = dataMapper.selectOne(dataQueryWrapper);
                        if (Float.parseFloat(energyData.getGasValue())!=0.0f) {
                            value = value + Float.parseFloat(energyData.getGasValue());
                            QueryWrapper<BasePigpen> queryWrapper1 = new QueryWrapper<>();
                            queryWrapper1.eq("id", pigpen.getParentId()).eq("farm_id", farmId);
                            BasePigpen basePigpen1 = basePigpenMapper.selectOne(queryWrapper1);
                            jsonObject.put("value", value);
                            jsonObject.put("buildName", basePigpen1.getBuildName());
                            jsonArray.add(jsonObject);
                            value = 0.00f;
                        }
                    }
                }
                JSONArray arraySort = jsonArraySort(jsonArray, "value", true);
                return new Result(ResultCode.SUCCESS, arraySort);
            }
            //本月
            else if ("2".equals(timeType)) {
                Date timesMonthmorning = DataUill.getTimesMonthmorning();
                basePigpenQueryWrapper.eq("parent_id", 0);
                List<BasePigpen> basePigpens = basePigpenMapper.selectList(basePigpenQueryWrapper);//得到所有的栋舍
                for (BasePigpen basePigpen : basePigpens) {
                    QueryWrapper<BasePigpen> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("farm_id", farmId).like("other2",basePigpen.getId());
                    List<BasePigpen> pigpens = basePigpenMapper.selectList(queryWrapper);//得到所有的楼层
                    Float value = 0.00f;
                    for (BasePigpen pigpen : pigpens) {
                        JSONObject jsonObject = new JSONObject();
                        QueryWrapper<EnergyData> dataQueryWrapper = new QueryWrapper<>();
                        dataQueryWrapper.eq("farm_id", farmId).eq("unit_id", pigpen.getId())
                                .ge("create_date", timesMonthmorning);
                        dataQueryWrapper.select(
                                "IFNULL(SUM(gas_value),0) gasValue  "
                        );
                        EnergyData energyData = dataMapper.selectOne(dataQueryWrapper);
                        if (Float.parseFloat(energyData.getGasValue())!=0.0f) {
                            value = value + Float.parseFloat(energyData.getGasValue());
                            QueryWrapper<BasePigpen> queryWrapper1 = new QueryWrapper<>();
                            queryWrapper1.eq("id", pigpen.getParentId()).eq("farm_id", farmId);
                            BasePigpen basePigpen1 = basePigpenMapper.selectOne(queryWrapper1);
                            jsonObject.put("value", value);
                            jsonObject.put("buildName", basePigpen1.getBuildName());
                            jsonArray.add(jsonObject);
                            value = 0.00f;
                        }
                    }
                }
                JSONArray arraySort = jsonArraySort(jsonArray, "value", true);
                return new Result(ResultCode.SUCCESS, arraySort);
            }
            //本周
            else if ("1".equals(timeType)) {
                Calendar calendar = Calendar.getInstance();
                calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
                basePigpenQueryWrapper.eq("parent_id", 0);
                basePigpenQueryWrapper.eq("parent_id", 0);
                List<BasePigpen> basePigpens = basePigpenMapper.selectList(basePigpenQueryWrapper);//得到所有的栋舍
                for (BasePigpen basePigpen : basePigpens) {
                    QueryWrapper<BasePigpen> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("farm_id", farmId).like("other2",basePigpen.getId());
                    List<BasePigpen> pigpens = basePigpenMapper.selectList(queryWrapper);//得到所有的楼层
                    Float value = 0.00f;
                    for (BasePigpen pigpen : pigpens) {
                        JSONObject jsonObject = new JSONObject();
                        QueryWrapper<EnergyData> dataQueryWrapper = new QueryWrapper<>();
                        dataQueryWrapper.eq("farm_id", farmId).eq("unit_id", pigpen.getId())
                                .ge("create_date", calendar.getTime());
                        dataQueryWrapper.select(
                                "IFNULL(SUM(gas_value),0) gasValue  "
                        );
                        EnergyData energyData = dataMapper.selectOne(dataQueryWrapper);
                        if (Float.parseFloat(energyData.getGasValue())!=0.0f) {
                            value = value + Float.parseFloat(energyData.getGasValue());
                            QueryWrapper<BasePigpen> queryWrapper1 = new QueryWrapper<>();
                            queryWrapper1.eq("id", pigpen.getParentId()).eq("farm_id", farmId);
                            BasePigpen basePigpen1 = basePigpenMapper.selectOne(queryWrapper1);
                            jsonObject.put("value", value);
                            jsonObject.put("buildName", basePigpen1.getBuildName());
                            jsonArray.add(jsonObject);
                            value = 0.00f;
                        }
                    }
                }
                JSONArray arraySort = jsonArraySort(jsonArray, "value", true);
                return new Result(ResultCode.SUCCESS, arraySort);
            }
        }


        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result listDataScreen(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        QueryWrapper<EnergyData> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId);
        List<EnergyAllDataVo> energyGasDataVos = dataMapper.listDayGas(queryWrapper);
        List<EnergyAllDataVo> energyFeedDataVos = dataMapper.listDayFeed(queryWrapper);
        List<EnergyAllDataVo> energyWaterDataVos = dataMapper.listDayWater(queryWrapper);
        List<EnergyAllDataVo> energyElectricityDataVos = dataMapper.listDayElectricity(queryWrapper);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("gas", energyGasDataVos);
        jsonObject.put("water", energyWaterDataVos);
        jsonObject.put("energyElectricity", energyElectricityDataVos);
        jsonObject.put("feed", energyFeedDataVos);
        return new Result(ResultCode.SUCCESS,jsonObject);
    }


    /*数组排序*/
    public static JSONArray jsonArraySort(JSONArray jsonArr, String sortKey, boolean is_desc) {
        //存放排序结果json数组
        JSONArray sortedJsonArray = new JSONArray();
        //用于排序的list
        List<JSONObject> jsonValues = new ArrayList<JSONObject>();
        //将参数json数组每一项取出，放入list
        for (int i = 0; i < jsonArr.size(); i++) {
            jsonValues.add(jsonArr.getJSONObject(i));
        }
        //快速排序，重写compare方法，完成按指定字段比较，完成排序
        Collections.sort(jsonValues, new Comparator<JSONObject>() {
            //排序字段
            private final String KEY_NAME = sortKey;

            //重写compare方法
            @Override
            public int compare(JSONObject a, JSONObject b) {
                //如果用String接会导致一位数和两位数混合比对的时候不能准确比出来，要用int类型接
                //String valA = new String();
                //String valB = new String();
                Float valA = 0.00f;
                Float valB = 0.00f;
                try {
                    valA = Float.parseFloat(a.getString(KEY_NAME));
                    valB = Float.parseFloat(b.getString(KEY_NAME));
                } catch (JSONException e) {
                    e.printStackTrace();
                }
                //是升序还是降序
                if (is_desc) {
                    return -valA.compareTo(valB);
                } else {
                    return -valB.compareTo(valA);
                }

            }
        });
        //将排序后结果放入结果jsonArray
        for (int i = 0; i < jsonArr.size(); i++) {
            sortedJsonArray.add(jsonValues.get(i));
        }
        return sortedJsonArray;
    }

}
